import json

import pytest

from zenroom import zenroom
from zenroom.zenroom import ZenroomException

LOAD_FATIGUE = 100


def test_basic():
    script = "print('Hello world')"
    output, errors = zenroom.zenroom_exec(script)

    assert "Hello world\n" == output


def test_keygen():
    script = """
    -- generate a simple keyring
    keyring = ECDH.new()
    keyring:keygen()
    
    -- export the keypair to json
    export = JSON.encode(
       {
          public  = keyring: public():base64(),
          private = keyring:private():base64()
       }
    )
    print(export)
    """
    output, _ = zenroom.zenroom_exec(script, verbosity=3)
    result = json.loads(output)
    assert 'public' in result
    assert 'private' in result


def test_zencode():
    contract = """Scenario 'coconut': "To run over the mobile wallet the first time and store the output as keypair.keys"
Given that I am known as 'identifier'
When I create my new keypair
Then print all data
    """

    result, _ = zenroom.zencode_exec(contract)
    assert result
    assert 'public' in result
    assert 'private' in result


def test_broken_script():
    with pytest.raises(ZenroomException) as e:
        script = "print('"
        result, errors = zenroom.zenroom_exec(script)

        assert e
        assert "line 1" in e


def test_broken_zencode():
    contract = """Scenario 'coconut': "broken"
    Given that I am known as '
    When I create my new keypair
    Then print all data
    """
    result, error = zenroom.zencode_exec(contract, verbosity=3)
    assert error
    assert "{}\n" == result


def test_random():
    script = """rng = RNG.new()
    buf = rng:octet(16)
    print(buf)
    """
    results = []
    for _ in range(LOAD_FATIGUE):
        result, error = zenroom.zenroom_exec_rng(script=script, random_seed=bytearray("", "utf=8"))
        results.append(result)
    assert len(set(results)) == len(results)


def test_random_zencode():
    script = """Scenario 'coconut': "test"
    Given that I am known as 'identifier'
    When I create my new keypair
    Then print all data
    """
    results = []
    for _ in range(LOAD_FATIGUE):
        result, error = zenroom.zencode_exec_rng(script=script, random_seed=bytearray("", "utf=8"))
        results.append(result)
    assert len(set(results)) == len(results)


def test_load_test():
    contract = """Scenario 'coconut': "To run over the mobile wallet the first time and store the output as keypair.keys"
    Given that I am known as 'identifier'
    When I create my new keypair
    Then print all data
        """

    for _ in range(LOAD_FATIGUE):
        print(f"#{_} CONTRACT")
        result, _ = zenroom.zencode_exec(contract)
        assert 'private' in result


def test_load_script():
    contract = """-- 0 for silent logging
ZEN:begin(0)

ZEN:parse([[
Scenario 'coconut': "To run over the mobile wallet the first time and store the output as keypair.keys"
Given that I am known as 'identifier'
When I create my new keypair
Then print all data
]])

ZEN:run()
    """

    for _ in range(LOAD_FATIGUE):
        print(f"#{_} CONTRACT")
        result, _ = zenroom.zenroom_exec(contract)
        assert 'private' in result




import click

from zenroom import zenroom


@click.command()
@click.option("-d", "--data", help="Data passed to zenroom", type=click.File("rb"))
@click.option("-k", "--keys", help="Keys passed to zenroom", type=click.File("rb"))
@click.argument("script", type=click.File("rb"), default="-", required=True)
def main(data, keys, script):
    data = data.read().decode() if data else None
    keys = keys.read().decode() if keys else None
    script = script.read().decode()
    result, _ = zenroom.zencode_exec(script=script, data=data, keys=keys)
    print(result)
    print(_)



from queue import Empty

from capturer import CaptureOutput

from .zenroom_swig import zenroom_exec_tobuf, zencode_exec_tobuf, zencode_exec_rng_tobuf, zenroom_exec_rng_tobuf
from multiprocessing import Process, Queue


__MAX_STRING__ = 1048576


class ZenroomException(Exception):
    pass


def _sanitize_output(out, err):
    out = out.decode().replace('\x00', '')
    err = err.decode().replace('\x00', '')
    return out, err


def _execute(func, queue, args):
    args['stdout_buf'] = bytearray(__MAX_STRING__)
    args['stderr_buf'] = bytearray(__MAX_STRING__)
    func(*args.values())
    queue.put_nowait(_sanitize_output(args['stdout_buf'], args['stderr_buf']))


def _zen_call(func, arguments):
    with CaptureOutput() as capturer:
        result = Queue()
        p = Process(target=_execute, args=(func, result, arguments))
        p.start()
        p.join()
        p.terminate()

    if result.empty():
        capturer.finish_capture()
        raise ZenroomException(capturer.get_lines())

    return result.get_nowait()


def zencode_exec(script, keys=None, data=None, conf=None, verbosity=1):

    """Invoke Zenroom, capturing and returning the output as a byte string
    This function is the primary method we expose from this wrapper library,
    which attempts to make Zenroom slightly simpler to call from Python. This
    wrapper has only been developed for a specific pilot project within DECODE,
    so beware - the code within this wrapper may be doing very bad things that
    the underlying Zenroom tool does not require.
    Args:
        script (str): Required byte string containing script which Zenroom will execute
        keys (str): Optional byte string containing keys which Zenroom will use
        data (str): Optional byte string containing data upon which Zenroom will operate
        conf (str): Optional byte string containing conf data for Zenroom
        verbosity (int): Optional int which controls Zenroom's log verbosity ranging from 1 (least verbose) up to 3 (most verbose)
    Returns:
            tuple: The output from Zenroom expressed as a byte string, the eventual errors generated as a string
    """
    args = dict(script=script, keys=keys, data=data, conf=conf, verbosity=verbosity, stdout_buf=None, stderr_buf=None)
    return _zen_call(zencode_exec_tobuf, args)


def zenroom_exec(script, keys=None, data=None, conf=None, verbosity=1):

    """Invoke Zenroom, capturing and returning the output as a byte string
    This function is the primary method we expose from this wrapper library,
    which attempts to make Zenroom slightly simpler to call from Python. This
    wrapper has only been developed for a specific pilot project within DECODE,
    so beware - the code within this wrapper may be doing very bad things that
    the underlying Zenroom tool does not require.
    Args:
        script (str): Required byte string containing script which Zenroom will execute
        keys (str): Optional byte string containing keys which Zenroom will use
        data (str): Optional byte string containing data upon which Zenroom will operate
        conf (str): Optional byte string containing conf data for Zenroom
        verbosity (int): Optional int which controls Zenroom's log verbosity ranging from 1 (least verbose) up to 3 (most verbose)
    Returns:
            bytes: The output from Zenroom expressed as a byte string
    """
    args = dict(script=script, keys=keys, data=data, conf=conf, verbosity=verbosity, stdout_buf=None, stderr_buf=None)
    return _zen_call(zenroom_exec_tobuf, args)


def zenroom_exec_rng(script, random_seed, keys=None, data=None, conf=None, verbosity=1):
    args = dict(script=script, keys=keys, data=data, conf=conf, verbosity=verbosity, stdout_buf=None, stderr_buf=None, random_seed=random_seed)
    return _zen_call(zenroom_exec_rng_tobuf, args)


def zencode_exec_rng(script, random_seed, keys=None, data=None, conf=None, verbosity=1):
    args = dict(script=script,
                keys=keys,
                data=data,
                conf=conf,
                verbosity=verbosity,
                stdout_buf=None,
                stderr_buf=None,
                random_seed=random_seed)
    return _zen_call(zencode_exec_rng_tobuf, args)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="zenroom",
    version="1.0.2",
    author="Sam Mulube",
    author_email="sam@thingful.net",
    maintainer="Puria Nafisi Azizi",
    maintainer_email="puria@dyne.org",
    description="Python wrapper for the Zenroom virtual machine",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/DECODEproject/zenroom-py",
    packages=setuptools.find_packages(),
    package_data={"zenroom": ["libs/Linux/**/**", "libs/Darwin/**/**"]},
    setup_requires=["pytest-runner"],
    tests_require=["pytest", "codecov", "pytest-cov"],
    install_requires=["capturer==2.4"],
    python_requires=">=3.5",
    project_urls={
            'Zenroom': 'https://zenroom.dyne.org',
            'DECODE': 'https://decodeproject.eu',
    },
    entry_points={
        "console_scripts": [
            "zenroom-test = tests.zenroom_cli_test:main",
        ]
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Intended Audience :: Developers",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS :: MacOS X",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Security"
      ],
)
